#include "StdAfx.h"
#include "NPHTTPClient.h"

#include <atlfile.h>
#include <atlstr.h>

#define AgentName _T("NHN PHOTOMANAGER HTTP Retriever 1.0")
#define	BOUNDARY	_T("----=_NextPart_000_0009_01C3B2B7.2FE1C3B0")
#define	BUF_FILECHUNC	1024*4
#define	CHUNK_SIZE		8196
#define __SIZE_BUFFER   (1024*16)

CNPHTTPClient::CNPHTTPClient(void)
{
}

CNPHTTPClient::~CNPHTTPClient(void)
{
}
/*! \brief : get lastmodifieddate & contentlength */
BOOL CNPHTTPClient::GetLastUpdate(LPCTSTR pszcUrl, SYSTEMTIME* pSysTimeLastUpdate, long* plSize)
{
	BOOL bSuccess = FALSE;
	try {
		HINTERNET hIO = InternetOpen(AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
		if (hIO==NULL) return FALSE;
		HINTERNET hIS = InternetOpenUrl(hIO, pszcUrl, NULL, 0, 0, 0);
		if (hIS == NULL)
		{
			InternetCloseHandle(hIO);
			return FALSE;
		}
		SYSTEMTIME systime;
		DWORD dwTimeSize = sizeof(SYSTEMTIME);

		bSuccess = HttpQueryInfo(hIS, HTTP_QUERY_LAST_MODIFIED | HTTP_QUERY_FLAG_SYSTEMTIME , &systime, (LPDWORD)&dwTimeSize, 0);
		if (bSuccess)
		{
			if (pSysTimeLastUpdate)
			{
				CopyMemory(pSysTimeLastUpdate, &systime, sizeof(SYSTEMTIME));
			}
		}
		const int nMax = 1024;
		TCHAR szHeader[nMax];
		bSuccess = HttpQueryInfo(hIS, HTTP_QUERY_CONTENT_LENGTH, szHeader, (LPDWORD)&nMax, 0);
		if (bSuccess)
		{
			*plSize = _ttol(szHeader);
		}
		InternetCloseHandle(hIO);
		InternetCloseHandle(hIS);
	}
	catch( char *str )    
	{
		ATLTRACE("Caught some other exception: %s\n", str);
		bSuccess = FALSE;
	}
	catch( UINT e)
	{
		ATLTRACE("Caught some other exception: %d\n", e);
		bSuccess = FALSE;
	}
	catch(...)
	{
		ATLTRACE("Caught some other exception: ASDF\n");
		bSuccess = FALSE;
	}
	return bSuccess;
}
/*! \brief : GET command */
BOOL CNPHTTPClient::GetPage(LPCTSTR pszcUrl, BYTE* pBuf, DWORD dwMaxBuf, LPCTSTR pszcCookie)
{
	HINTERNET hIO = InternetOpen(AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
	if (hIO==NULL) return FALSE;
	if (pszcCookie!=NULL)
	{
		CString szURL = _T("http://");
		szURL += pszcUrl;
		::InternetSetCookie( szURL, NULL, pszcCookie);
	}
	HINTERNET hIS = InternetOpenUrl(hIO, pszcUrl, NULL, 0, HTTP_QUERY_DATE, 0);
	if (hIS == NULL)
	{
		InternetCloseHandle(hIO);
		return FALSE;
	}
	BOOL bSuccess = ReceiveData(hIS, pBuf, dwMaxBuf);	

	InternetCloseHandle(hIS);
	InternetCloseHandle(hIO);
	return bSuccess;
}
/*! \brief : POST command and download file */
BOOL CNPHTTPClient::DownloadFile(LPCTSTR pszcUrl, LPCTSTR pszcDownloadPath, DWORD_PTR dwParam, PFNHTTPSTATUSCALLBACK pfn)
{
	HINTERNET hIO = InternetOpen(AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
	if (hIO==NULL) return FALSE;
	
	HINTERNET hIS = InternetOpenUrl(hIO, pszcUrl, NULL, 0, HTTP_QUERY_DATE, 0);
	if (hIS == NULL)
	{
		InternetCloseHandle(hIO);
		return FALSE;
	}
	BOOL bSuccess = DownloadFile(hIS, pszcDownloadPath, dwParam, pfn);

	InternetCloseHandle(hIS);
	InternetCloseHandle(hIO);
	return bSuccess;
}
///////////////////////////////////////////////////////////////////////////////////////////
BOOL CNPHTTPClient::ReceiveData(HINTERNET hIS, BYTE* pBuf, DWORD dwMaxBuf)
{
	DWORD dwSize = 0;
	DWORD dwRead = 0, dwReadCount = 0;
	BYTE	bufRead[BUF_FILECHUNC];

	InternetQueryDataAvailable(hIS, &dwSize, 0, 0);
	InternetReadFile(hIS, bufRead, min(BUF_FILECHUNC, dwSize), &dwRead);
	while (dwRead != 0)
	{
		if (dwMaxBuf < dwReadCount+dwRead)
		{
			//overflow
			break;
		}
		CopyMemory(pBuf+dwReadCount, bufRead, dwRead);
		
		dwReadCount += dwRead;
		InternetQueryDataAvailable(hIS, &dwSize, 0, 0);
		InternetReadFile(hIS, bufRead, min(BUF_FILECHUNC, dwSize), &dwRead);
	}
	return TRUE;
}
BOOL CNPHTTPClient::DownloadFile(HINTERNET hIS, LPCTSTR pszcDownloadPath, DWORD_PTR dwParam, PFNHTTPSTATUSCALLBACK pfn)
{
	CAtlTemporaryFile	file;
	HRESULT hr = file.Create();
	if (FAILED(hr)) return FALSE;

	DWORD dwSize = 0;
	DWORD dwRead = 0, dwWritten = 0, dwReadCount = 0;
	BYTE	bufRead[BUF_FILECHUNC];

	ULONG lContentLen = 0;
	const int nMax = 1024;
	TCHAR szHeader[nMax] = {0, };	
	if (HttpQueryInfo(hIS, HTTP_QUERY_CONTENT_LENGTH, szHeader, (LPDWORD)&nMax, 0))
	{
		lContentLen = _ttol(szHeader);
	}
	InternetQueryDataAvailable(hIS, &dwSize, 0, 0);
	InternetReadFile(hIS, bufRead, min(BUF_FILECHUNC, dwSize), &dwRead);
	while (dwRead != 0)
	{
		hr = file.Write(bufRead, dwRead, &dwWritten);
		if (FAILED(hr)) break;

		dwReadCount += dwWritten;
		if (pfn)
		{
			BOOL bContinue = pfn(dwReadCount, lContentLen, dwParam);
			if (bContinue == FALSE) break;
		}
		InternetQueryDataAvailable(hIS, &dwSize, 0, 0);
		InternetReadFile(hIS, bufRead, min(BUF_FILECHUNC, dwSize), &dwRead);
	}
	hr = file.Close(pszcDownloadPath);
	return SUCCEEDED(hr);
}
BOOL CNPHTTPClient::PostPage(const TCHAR *strUrl, PMB_POSTARG_ARR pPostArg, LPCTSTR pszCookie, TCHAR* strPage, int nMaxPage, DWORD_PTR dwParam, PFNHTTPSTATUSCALLBACK lpCallback)
{
	HINTERNET hIO = InternetOpen(AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
	if (hIO==NULL) return FALSE;
	URL_COMPONENTS uc;
	//let's split the url...
	uc.dwStructSize = sizeof(uc);
	uc.lpszScheme = NULL;
	uc.dwSchemeLength = 0;
	uc.lpszHostName = NULL;
	uc.dwHostNameLength = 1;
	uc.nPort = 0;
	uc.lpszUserName = NULL;
	uc.dwUserNameLength = 0;
	uc.lpszPassword = NULL;
	uc.dwPasswordLength = 0;
	uc.lpszUrlPath = NULL;
	uc.dwUrlPathLength = 1;
	uc.lpszExtraInfo = NULL;
	uc.dwExtraInfoLength = 0;
	InternetCrackUrl(strUrl, _tcslen(strUrl), 0, &uc);
	//post the data...
	TCHAR *szHostName = _tcsdup(uc.lpszHostName);
	szHostName[uc.dwHostNameLength] = '\0';
	TCHAR *szFileName = _tcsdup(uc.lpszUrlPath);
	szFileName[uc.dwUrlPathLength] = '\0';
	DWORD flags;
	flags = INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE | INTERNET_FLAG_FORMS_SUBMIT;

	TCHAR szAccept[] = _T("*/*"); //we accept everything...
	LPTSTR AcceptTypes[2]={0};
	AcceptTypes[0]=szAccept;
	HINTERNET hCO = InternetConnect(hIO, szHostName, uc.nPort, _T(""), _T(""), INTERNET_SERVICE_HTTP,
		INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE,
		0);
	if (hCO == NULL)
	{
		InternetCloseHandle(hIO);
		return FALSE;
	}
	if (pszCookie!=NULL)
	{
		CString szURL = _T("http://");
		szURL += strUrl;
		BOOL bRes = ::InternetSetCookie( szURL, NULL, pszCookie);
	}
	HINTERNET hIS = HttpOpenRequest(hCO, _T("POST"), szFileName, NULL, NULL, (LPCTSTR*)AcceptTypes, flags, 0);
	if (hIS == NULL)
	{
		InternetCloseHandle(hCO);
		InternetCloseHandle(hIO);
		return FALSE;
	}
	TCHAR	szPostArguments[__SIZE_BUFFER]=_T("");	
	//Content-Type
	CString strContentType;
	strContentType.Format(_T("Content-Type: application/x-www-form-urlencoded\r\n"));
	BOOL bSuccess = HttpAddRequestHeaders( hIS, strContentType, strContentType.GetLength(),  HTTP_ADDREQ_FLAG_ADD_IF_NEW);
	//Content-Length
	TCHAR szContentLength[1024];
	DWORD dwPostBufferLength = GetUrlEncodedFromArg(szPostArguments, pPostArg);
	_stprintf(szContentLength, _T("Content-Length: %d\r\n"), dwPostBufferLength);
	bSuccess = HttpAddRequestHeaders( hIS, szContentLength, _tcslen(szContentLength),  HTTP_ADDREQ_FLAG_ADD_IF_NEW);

	bSuccess = HttpSendRequest( hIS,	// handle by returned HttpOpenRequest
		strContentType, // additional HTTP header
		_tcslen(strContentType), // additional HTTP header length
		(LPVOID)szPostArguments, // additional data in HTTP Post or HTTP Put
		_tcslen(szPostArguments)); // additional data length

	bSuccess = ReceiveData(hIS, strPage, nMaxPage);
	if (!bSuccess)
	{
		DWORD dwError = GetLastError();
		ATLTRACE("AdvHTTPClient %d\n", dwError);
	}
	free(szHostName);
	free(szFileName);

	InternetCloseHandle(hIS);
	InternetCloseHandle(hCO);
	InternetCloseHandle(hIO);
	return bSuccess;
}
DWORD CNPHTTPClient::GetUrlEncodedFromArg(LPTSTR pszContent, PMB_POSTARG_ARR pPostArgArr)
{
	MB_POSTARG_ARR arrPostArg = *pPostArgArr;
	USES_CONVERSION;
	int nCount = arrPostArg.GetSize();
	CString strContent;
	for(int i=0; i<nCount; i++)
	{
		PMB_POSTARG pPostArg = (PMB_POSTARG)arrPostArg[i];
		
		strContent.Append(CW2CT(pPostArg->bstrKey));
		strContent.AppendChar(_T('='));
		strContent.Append(CW2CT(pPostArg->bstrValue));
		if (i<nCount-1)
		{
			strContent.AppendChar(_T('&'));
		}
	}

	_tcscpy(pszContent, strContent);
	return strContent.GetLength();
}
BOOL CNPHTTPClient::UploadFile(const TCHAR *strUrl, PMB_POSTARG_ARR pPostArg, LPCTSTR pszCookie, TCHAR* strPage, int nMaxPage, DWORD_PTR dwParam, PFNHTTPSTATUSCALLBACK lpCallback)
{
	HINTERNET hIO = InternetOpen(AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
	if (hIO==NULL) return FALSE;
	URL_COMPONENTS uc;
	//let's split the url...
	uc.dwStructSize = sizeof(uc);
	uc.lpszScheme = NULL;
	uc.dwSchemeLength = 0;
	uc.lpszHostName = NULL;
	uc.dwHostNameLength = 1;
	uc.nPort = 0;
	uc.lpszUserName = NULL;
	uc.dwUserNameLength = 0;
	uc.lpszPassword = NULL;
	uc.dwPasswordLength = 0;
	uc.lpszUrlPath = NULL;
	uc.dwUrlPathLength = 1;
	uc.lpszExtraInfo = NULL;
	uc.dwExtraInfoLength = 0;
	InternetCrackUrl(strUrl, _tcslen(strUrl), 0, &uc);
	//post the data...
	TCHAR *szHostName = _tcsdup(uc.lpszHostName);
	szHostName[uc.dwHostNameLength] = '\0';
	TCHAR *szFileName = _tcsdup(uc.lpszUrlPath);
	szFileName[uc.dwUrlPathLength] = '\0';
	DWORD flags;
	flags = INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE | INTERNET_FLAG_FORMS_SUBMIT;

	TCHAR szAccept[] = _T("*/*"); //we accept everything...
	LPTSTR AcceptTypes[2]={0};
	AcceptTypes[0]=szAccept;
	HINTERNET hCO = InternetConnect(hIO, szHostName, uc.nPort, _T(""), _T(""), INTERNET_SERVICE_HTTP,
		INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE,
		0);
	if (hCO == NULL)
	{
		InternetCloseHandle(hIO);
		return FALSE;
	}
	if (pszCookie!=NULL)
	{
		CString szURL = _T("http://");
		szURL += strUrl;
		BOOL bRes = ::InternetSetCookie( szURL, NULL, pszCookie);
	}
	HINTERNET hIS = HttpOpenRequest(hCO, _T("POST"), szFileName, NULL, NULL, (LPCTSTR*)AcceptTypes, flags, 0);
	if (hIS == NULL)
	{
		InternetCloseHandle(hCO);
		InternetCloseHandle(hIO);
		return FALSE;
	}
	//Content-Type
	CString strContentType;
	strContentType.Format(_T("Content-Type: multipart/form-data; boundary=%s\r\n"), BOUNDARY);
	BOOL bSuccess = HttpAddRequestHeaders( hIS, strContentType, strContentType.GetLength(),  HTTP_ADDREQ_FLAG_ADD_IF_NEW);
	//Content-Length
	TCHAR szContentLength[1024];
	DWORD dwPostBufferLength = GetMultiPartsFormDataLength(pPostArg);
	_stprintf(szContentLength, _T("Content-Length: %d\r\n"), dwPostBufferLength);
	bSuccess = HttpAddRequestHeaders( hIS, szContentLength, _tcslen(szContentLength),  HTTP_ADDREQ_FLAG_ADD_IF_NEW);

	INTERNET_BUFFERS InternetBufferIn={0};
	InternetBufferIn.dwStructSize=sizeof(INTERNET_BUFFERS);
	InternetBufferIn.Next=NULL;		

	bSuccess = HttpSendRequestEx(hIS, &InternetBufferIn, NULL, HSR_INITIATE, 0);
	SendMultiPartsFormData(hIS, pPostArg, dwParam, lpCallback);
	bSuccess = HttpEndRequest(hIS, NULL, HSR_INITIATE, 0);

	bSuccess = ReceiveData(hIS, strPage, nMaxPage);
	if (!bSuccess)
	{
		DWORD dwError = GetLastError();
		ATLTRACE("AdvHTTPClient %d\n", dwError);
	}
	free(szHostName);
	free(szFileName);

	InternetCloseHandle(hIS);
	InternetCloseHandle(hCO);
	InternetCloseHandle(hIO);
	return bSuccess;
}

DWORD CNPHTTPClient::SendMultiPartsFormData(HINTERNET hRO, PMB_POSTARG_ARR pPostArgArr, DWORD_PTR dwParam, PFNHTTPSTATUSCALLBACK pfn)
{
	MB_POSTARG_ARR arrPostArg = *pPostArgArr;
	DWORD	dwLength = 0;
	DWORD	dwWritten = 0;
	USES_CONVERSION;
	int nCount = arrPostArg.GetSize();
	TCHAR buf[8096];
	for(int i=0; i<nCount; i++)
	{
		PMB_POSTARG pPostArg = (PMB_POSTARG)arrPostArg[i];
		TCHAR szKey[1024] = {0};
		TCHAR szValue[1024] = {0};
		_tcscpy(szKey, CW2CT(pPostArg->bstrKey));
		_tcscpy(szValue, CW2CT(pPostArg->bstrValue));

		if (pPostArg->bBinary)
		{
			_stprintf( (TCHAR*)buf,
				_T("--%s\r\n")
				_T("Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"\r\n")
				_T("Content-Type: %s\r\n")
				_T("\r\n"),
				BOUNDARY,
				szKey,
				szValue,
				GetContentType(szValue)
				);
			InternetWriteFile(hRO, buf, _tcslen(buf), &dwWritten);
			dwLength += dwWritten;
			dwWritten = 0;

			DWORD dwTotalBytes = 0;
			HANDLE hFile=::CreateFile(szValue, 
				GENERIC_READ, // desired access
				FILE_SHARE_READ, // share mode
				NULL, // security attribute
				OPEN_EXISTING, // create disposition
				FILE_ATTRIBUTE_NORMAL, // flag and attributes
				NULL); // template file handle
			DWORD dwTotalRead = 0;
			if (hFile != INVALID_HANDLE_VALUE)
			{
				DWORD dwContentLen = 0;
				dwContentLen = GetFileSize(hFile, NULL);
				TCHAR bufFile[BUF_FILECHUNC];
				DWORD dwRead = 0;
				while (ReadFile(hFile, bufFile, BUF_FILECHUNC, &dwRead, NULL))
				{
					InternetWriteFile(hRO, bufFile, dwRead, &dwWritten);
					if (dwRead != dwWritten)
					{
						break;
					}
					dwTotalRead += dwRead;

					if (pfn)
					{
						BOOL bContinue = pfn(dwTotalRead, dwContentLen, dwParam);
						/*CString strMsg;
						strMsg.Format(_T("SendMultiPartsFormData i(%d), bContinue(%d)\n"), i, bContinue);
						OutputDebugString(strMsg);*/
						if (bContinue == FALSE) break;
					}
				}
				::CloseHandle(hFile);			
			}	
			InternetWriteFile(hRO, _T("\r\n"), _tcslen(_T("\r\n")), &dwWritten);
			dwLength += dwTotalRead + _tcslen(_T("\r\n"));				
		}
		else
		{
			_stprintf(	buf,						
				_T("--%s\r\n")
				_T("Content-Disposition: form-data; name=\"%s\"\r\n")
				_T("\r\n")
				_T("%s\r\n"),
				BOUNDARY,
				szKey,
				szValue
				);
			InternetWriteFile(hRO, buf, _tcslen(buf), &dwWritten);
			dwLength += dwWritten;
		}
	}
	InternetWriteFile(hRO, _T("--"), 2, &dwWritten);
	InternetWriteFile(hRO, BOUNDARY, _tcslen(BOUNDARY), &dwWritten);
	InternetWriteFile(hRO, _T("--\r\n"), 3, &dwWritten);

	return dwLength+5+_tcslen(BOUNDARY);
}
DWORD CNPHTTPClient::GetMultiPartsFormDataLength(PMB_POSTARG_ARR pPostArgArr)
{
	MB_POSTARG_ARR arrPostArg = *pPostArgArr;
	DWORD	dwLength = 0;
	USES_CONVERSION;
	int nCount = arrPostArg.GetSize();
	TCHAR buf[8096];
	for(int i=0; i<nCount; i++)
	{
		PMB_POSTARG pPostArg = (PMB_POSTARG)arrPostArg[i];
		TCHAR szKey[1024] = {0};
		TCHAR szValue[1024] = {0};
		_tcscpy(szKey, CW2CT(pPostArg->bstrKey));
		_tcscpy(szValue, CW2CT(pPostArg->bstrValue));

		if (pPostArg->bBinary)
		{
			_stprintf( (TCHAR*)buf,
				_T("--%s\r\n")
				_T("Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"\r\n")
				_T("Content-Type: %s\r\n")
				_T("\r\n"),
				BOUNDARY,
				szKey,
				szValue,
				GetContentType(szValue)
				);

			dwLength += _tcslen(buf);
			DWORD dwTotalBytes = 0;
			HANDLE hFile=::CreateFile(OLE2T(pPostArg->bstrValue), 
				GENERIC_READ, // desired access
				FILE_SHARE_READ, // share mode
				NULL, // security attribute
				OPEN_EXISTING, // create disposition
				FILE_ATTRIBUTE_NORMAL, // flag and attributes
				NULL); // template file handle
			if (hFile != INVALID_HANDLE_VALUE)
			{
				dwLength += ::GetFileSize(hFile, NULL) + _tcslen(_T("\r\n"));
				::CloseHandle(hFile);			
			}
		}
		else
		{			
			_stprintf(	buf,						
				_T("--%s\r\n")
				_T("Content-Disposition: form-data; name=\"%s\"\r\n")
				_T("\r\n")
				_T("%s\r\n"),
				BOUNDARY,
				szKey,
				szValue
				);
			dwLength += _tcslen(buf);
		}
	}
	return dwLength+5+_tcslen(BOUNDARY);
}
LPCTSTR CNPHTTPClient::GetContentType(LPCTSTR szName)
{

	static TCHAR	szReturn[1024]= _T("");
	LONG	dwLen=1024;
	DWORD	dwDot=0;

	for(dwDot=_tcslen(szName);dwDot>0;dwDot--)
	{
		if(!_tcsncmp((szName+dwDot), _T("."), 1))
			break;
	}

	HKEY	hKEY;
	LPTSTR	szWord=(TCHAR*)(szName+dwDot);

	if(RegOpenKeyEx(HKEY_CLASSES_ROOT, szWord, 0, KEY_QUERY_VALUE, &hKEY)==ERROR_SUCCESS)
	{
		if(RegQueryValueEx(hKEY, TEXT("Content Type"), NULL, NULL, (LPBYTE)szReturn, (unsigned long*)&dwLen)!=ERROR_SUCCESS)
			_tcsncpy(szReturn, _T("application/octet-stream"), _tcslen(_T("application/octet-stream")));
		RegCloseKey(hKEY);
	}
	else
	{
		_tcsncpy(szReturn, _T("application/octet-stream"), _tcslen(_T("application/octet-stream")));
	}

	return szReturn;
}
BOOL CNPHTTPClient::ReceiveData(HINTERNET hIS, TCHAR* strPage, int nMaxPage)
{
	TCHAR	*szReceivedData = NULL;
	char bufChunk[CHUNK_SIZE];
	DWORD dwRead = 0;
	int nPos = 0;
	szReceivedData = (TCHAR*)calloc(CHUNK_SIZE + 1, sizeof(TCHAR));
	while (InternetReadFile(hIS, bufChunk, CHUNK_SIZE - 1, &dwRead))
	{
		if (dwRead == 0) break;
		bufChunk[dwRead] = '\0';
		nPos += dwRead;
		szReceivedData[nPos] = '\0';
		CopyMemory(szReceivedData, bufChunk, dwRead);
		szReceivedData = (TCHAR*)realloc(szReceivedData, nPos + dwRead);
	}
	if (strPage != NULL && nMaxPage > 0)
	{
		_tcsncpy(strPage, szReceivedData, nMaxPage);
	}
	free(szReceivedData);
	//return (nPos>0);
	return TRUE;
}
/************************************************************************/
/* CNPPostArg                                                                
/************************************************************************/
void CNPPostArg::AddArgument(LPCTSTR strKey, LPCTSTR strValue)
{
	PMB_POSTARG pPostArg;
	if (NewPostArg(&pPostArg))
	{
		USES_CONVERSION;
		pPostArg->bstrKey = SysAllocString(T2CW(strKey));
		pPostArg->bstrValue = SysAllocString(T2CW(strValue));
		pPostArg->bBinary = FALSE;

		m_arrPostArg.Add(pPostArg);
	}
}
void CNPPostArg::AddFile(LPCTSTR strKey, LPCTSTR strFilePath)
{
	PMB_POSTARG pPostArg;
	if (NewPostArg(&pPostArg))
	{
		USES_CONVERSION;
		pPostArg->bstrKey = SysAllocString(T2CW(strKey));
		pPostArg->bstrValue = SysAllocString(T2CW(strFilePath));
		pPostArg->bBinary = TRUE;

		m_arrPostArg.Add(pPostArg);
	}
}
void CNPPostArg::RemoveAll()
{
	int nCount = m_arrPostArg.GetSize();
	for (int i=0; i<nCount; i++)
	{
		PMB_POSTARG pPostArg = m_arrPostArg[i];
		DeletePostArg(pPostArg);
	}
	m_arrPostArg.RemoveAll();
}